/*======================================================================*\
|| #################################################################### ||
|| # vBulletin [#]version[#]
|| #################################################################### ||
\*======================================================================*/

// #############################################################################
// vB_Text_Editor

/**
* vBulletin Editor Class
*
* Activates any HTML controls for an editor
*
*
* @param	string	Unique key for this editor
* @param	boolean	Initialise to WYSIWYG mode?
* @param	string	Unique token for this editor
* @param	boolean	Auto save content?
* @param	string	Editor name
* @param	string	(Optional) Extra arguments to pass when switching editor modes
*/
function vB_Text_Editor(editorid, mode, editortoken, autosave, editorname)
{
	/**
	* Miscellaneous variables
	*/
	this.editorid = editorid;
	this.editorname = editorname || '';
	this.mode = parseInt(mode, 10);
	this.initialized = false;
	this.editortoken = editortoken || 'nonforum';
	this.textobj = fetch_object(this.editorid + '_textarea');
	this.unchanged = this.textobj.value;
	this.options = editoroptions || {};
	this.submitready = false;

	this.autosave_on = this.mode && !!autosave;
	this.smilie_on = this.mode && (typeof this.options.smilies != 'undefined') && this.options.smilies;
	this.mentions_on = this.mode && (typeof this.options.mentions != 'undefined') && this.options.mentions;
	this.attach_on = this.mode && (typeof this.options.attach != 'undefined') && this.options.attach;

	// =============================================================================
	// vB_Text_Editor methods

	/**
	* Editor initialization wrapper
	*/
	this.init = function()
	{
		if (this.initialized)
		{
			return;
		}

		this.init_controls();

		this.captcha = document.getElementById("imagestamp");
		if (this.captcha != null)
		{
			this.captcha.setAttribute("tabIndex", 1);
		}

		this.initialized = true;
	}

	/**
	* Init button controls for the editor
	*/
	this.init_controls = function()
	{
		if (this.mode)
		{
			// Currently tables are not supported
			// ToDo: Add support for tables
			CKEDITOR.dtd.body.table = 0;

			var config = {
				// customConfig: 'config.js'
				disableNativeSpellChecker: false,
				pasteFilter: content_filter()
			};
			var removeButtons = [];

			if (this.options.show)
			{
				// ToDo: Add support font and size bbcode
				/*
				if (!this.options.show.font_bbcode)
				{
					removeButtons.push('Font');
				}

				if (!this.options.show.size_bbcode)
				{
					removeButtons.push('FontSize');
				}
				*/

				if (!this.options.show.color_bbcode)
				{
					removeButtons.push('TextColor');
				}

				if (!this.options.show.basic_bbcode)
				{
					removeButtons.push('Bold', 'Italic', 'Underline');
				}

				if (!this.options.show.align_bbcode)
				{
					removeButtons.push('JustifyCenter', 'JustifyLeft', 'JustifyRight', 'JustifyBlock');
				}

				if (!this.options.show.list_bbcode)
				{
					removeButtons.push('NumberedList', 'BulletedList');
				}

				if (!this.options.show.align_bbcode && !this.options.show.list_bbcode)
				{
					removeButtons.push('Outdent', 'Indent');
				}

				if (!this.options.show.url_bbcode)
				{
					removeButtons.push('Link', 'Unlink');
				}

				if (!this.options.show.img_bbcode)
				{
					removeButtons.push('Image');
				}

				if (!this.options.show.quote_bbcode)
				{
					removeButtons.push('Blockquote');
				}

				// ToDo: Add support code, html & php bbcode
				/*
				if (!this.options.show.code_bbcode && !this.options.show.html_bbcode && !this.options.show.php_bbcode)
				{
					removeButtons.push('CodeSnippet');
				}
				*/
			}

			if (this.autosave_on)
			{
				config.autosave = {
					'SaveKey' : COOKIE_PREFIX + '_autosave_' + this.editortoken,
					'messageType': 'statusbar',
					'autoLoad': false,
					'restoreDiff': false,
					'saveDetectionSelectors': false,
					'onSaveData': this.autosave_save_data.bind(this),
					'onRestoreData': this.autosave_restore_data.bind(this),
					'onRemoveData': this.autosave_remove_data.bind(this)
				};

				if (typeof vBulletinStorage !== 'undefined')
				{
					if (vBulletinStorage.webstoragetype == 3)
					{
						config.autosave.encodeData = false;
					}
					config.autosave.storage = vBulletinStorage;
				}

				// Enable title autosave
				this.autosave_title = autosave_title_elem(this.editorid);
			}
			else
			{
				// Disable autosave
				config.autosave = false;
			}

			if (this.smilie_on)
			{
				config.smiley_descriptions = this.options.smilies.smiley_descriptions;
				config.smiley_images = this.options.smilies.smiley_images;
				config.smiley_path = this.options.smilies.smiley_path;
				config.smiley_map = this.options.smilies.smiley_map;
				config.smiley_columns = this.options.smilies.smiley_columns;
			}
			else
			{
				removeButtons.push('Smiley');
			}

			if (this.mentions_on)
			{
				var utagmarker = '@',
					pattern = '\\' + utagmarker + '\\S[^\\r\\n\\t\\f\\v]',
					url = window.location.href.split('/');

				if (this.options.mentions.maxchars)
				{
					pattern += '{1,' + this.options.mentions.maxchars + '}';
				}
				else
				{
					pattern += '{1,}';
				}
				pattern += '$';

				config.mentions = [
					// user tagging
					{
						feed: this.mentions_feed,
						pattern: new RegExp(pattern),
						itemTemplate: '<li data-id="{id}"><div class="cke_utag_avatar" style="background-image:url(\'{avatar}\')"></div><strong class="username">{username}</strong></li>',
						outputTemplate: '<a href="{profileurl}" class="utag">' + utagmarker + '{username}</a>&nbsp;'
					}
				];

				config.utag_marker = utagmarker;
				config.utag_replace = 'member.php?u=$1';
			}

			// We need this option setup even if this editor doesn't have attachments on.
			config.inlineattach_urlmap = 'attachment.php?attachmentid=$1&d=' + Date.now();

			if (this.attach_on)
			{
				config.inlineattach_items = this.options.attach.items || [];
				config.inlineattach_size = this.options.attach.thumbs || 0;
				config.inlineattach_urlmap = this.options.attach.urlmap || config.inlineattach_urlmap;
			}
			else
			{
				removeButtons.push('InlineAttach');
			}

			// Extra plugins
			if (this.options.plugins)
			{
				config.extraPlugins = this.options.plugins.join();
			}

			// Use DivArea only on iOS
			// https://github.com/ckeditor/ckeditor4/issues/498
			// https://bugs.webkit.org/show_bug.cgi?id=184178
			if (!CKEDITOR.env.iOS)
			{
				config.removePlugins = 'divarea';
			}

			var editor = CKEDITOR.replace(this.editorid + '_textarea', config);
			// Fired once the editor configuration is ready (loaded and processed)
			editor.on('configLoaded', function() {
				editor.config.removeButtons += (',' + removeButtons.join());
			});
			// Fired when the editor instance is completely created, fully initialized and ready for interaction
			editor.on('instanceReady', function() {
				if (this.options.focus) {
					this.focus_break();
				}
			}, this);

			this.editor = editor;
		}
	}

	this.update_controls = function(config)
	{
		if (this.mode)
		{
			this.editor.on('configLoaded', function(evt) {
				// merge config into editor.config
				jQuery.extend(evt.editor.config, config);
			});
		}
	}

	// =============================================================================
	// Standard editor
	// =============================================================================
	/**
	* Prepare Form For Submit
	*/
	this.prepare_submit = function(subjecttext, minchars)
	{
		if (this.mode && this.check_dirty())
		{
			this.textobj.value = this.editor.getData();
		}

		var returnvalue = validatemessage(this.textobj.value, subjecttext, minchars);

		this.submitready = returnvalue;

		if (returnvalue)
		{
			if (this.autosave_on)
			{
				this.editor.execCommand('saveDetection');
			}
			return returnvalue;
		}
		else if (this.captcha != null && this.captcha.failed)
		{
			return returnvalue;
		}
		else
		{
			return false;
		}
	}

	/**
	* Writes contents to the <textarea>
	*
	* @param	object	<textarea>
	* @param	string	Initial text
	*/
	this.write_editor_contents = function(txt)
	{
		this.textobj.value = txt;

		if (this.mode)
		{
			CKEDITOR.instances[this.editorid + '_textarea'].setData(txt);
		}
	}

	/**
	* Get Editor Contents
	*/
	this.get_editor_contents = function()
	{
		if (this.mode)
		{
			this.textobj.value = this.editor.getData();
		}

		return this.textobj.value;
	};

	/**
	* Checks whether the current editor content contains changes
	*/
	this.check_dirty = function()
	{
		var currenttxt = this.textobj.value && this.textobj.value.trim();

		if (this.mode)
		{
			// ACES-13015 CKEditor's checkDirty function works correctly on most browsers but Webkit
			return this.editor.checkDirty() && (!CKEDITOR.env.webkit || currenttxt != this.editor.getData().trim());
		}
		else
		{
			return !!currenttxt && (currenttxt != this.unchanged.trim());
		}
	}

	/**
	* Sets the width of the editor. Works around an IE issue if necessary.
	*
	* @param	string	Width to set
	* @param	bool	Whether to overwrite the "original" width setting
	*/
	this.set_editor_width = function(width)
	{
		this.textobj.style.width = width;
	}

	/**
	* Destroy Editor
	*/
	this.destroy = function()
	{
		if (this.mode)
		{
			CKEDITOR.instances[this.editorid + '_textarea'].destroy();
		}
	};

	/**
	* Prepare Attachments Popup
	*
	* @param	fn	Function to call
	*/
	this.open_attachments_popup = function(fn)
	{
		if (this.mode)
		{
			this.editor.config.inlineattach_onopen = fn;
		}
	};

	/**
	* Build Attachments Popup
	*
	* @param	object	The list of attachments
	*/
	this.build_attachments_popup = function(attachments, thumbssize)
	{
		if (this.mode && this.editor.status == 'ready')
		{
			var popupList = [];

			for (var id in attachments)
			{
				popupList.push({
					'id': id,
					'filename': attachments[id]['filename'],
					'imgpath': attachments[id]['imgpath'],
					'extension': attachments[id]['extension'],
					'source': attachments[id]['source']
				});
			}

			this.editor.execCommand('updateAttach', popupList);
		}
	};

	/**
	* Wrap Tags
	*
	* @param	string	Tag to wrap
	* @param	boolean	Use option?
	* @param	string	(Optional) selected text
	*
	* @return	boolean
	*/
	this.wrap_tags = function(tagname, useoption, selection)
	{
		tagname = tagname.toUpperCase();

		if (typeof selection == 'undefined')
		{
			selection = '';
		}

		var opentag;
		if (useoption !== false)
		{
			opentag = '[' + tagname + '="' + useoption + '"' + ']';
		}
		else
		{
			opentag = '[' + tagname + ']';
		}

		var closetag = '[/' + tagname + ']';
		var text = opentag + selection + closetag;

		this.editor.insertText(text);

		return false;
	};

	/**
	* Prepares text to be used in the editor
	*/
	this.clean_text = function(txt)
	{
		return jQuery('<textarea />').html(txt).text();
	};

	/**
	* Paste HTML
	*/
	this.insert_text = function(str, html)
	{
		if (typeof html == 'undefined')
		{
			html = false;
		}

		if (this.mode)
		{
			if (this.editor.mode == 'source')
			{
				alert('Please switch your editor to WYSIWYG mode.');
				return false;
			}

			str = this.clean_text(str);

			if (html)
			{
				var data = this.editor.fire('setData', {'dataValue': str});

				this.editor.insertHtml(data.dataValue);
			}
			else
			{
				this.editor.insertText(str);
			}

			return true;
		}

		return false;
	};

	/**
	* Focus in the editor after the instance is ready
	*/
	this.focus_break = function()
	{
		if (this.mode)
		{
			// Focus the editor, otherwise nothing will happen
			this.editor.focus();

			if (this.editor.getData() != '')
			{
				// selecting the starting node and move the cursor
				// to the end of the range
				var range = this.editor.createRange();
				range.moveToElementEditEnd(range.root);
				this.editor.getSelection().selectRanges([range]);
				// simulate a carriage return
				this.editor.fire('lockSnapshot');
				this.editor.execCommand('enter');
				this.editor.fire('unlockSnapshot');
				this.editor.resetDirty();
			}
		}
	};

	// =============================================================================
	// Mentions plugin
	// =============================================================================
	if (this.mentions_on)
	{
		this.mentions_feed = function(options, callback)
		{
			var term = options.query;
			var data = {
				'do': 'usersearchjson',
				'securitytoken': SECURITYTOKEN,
				'fragment': term
			};

			jQuery
				.post('ajax.php', data, null, 'json')
				.done(function(res, evt) {
					if (evt === 'success') {
						callback(res);
					}
				});
		};
	}

	// =============================================================================
	// AutoSave plugin
	// =============================================================================
	if (this.autosave_on)
	{
		// Checks if there is an input with data attribute "autosave" or name "title"
		function autosave_title_elem(editorid)
		{
			var parentform = jQuery('#' + editorid + '_textarea').parents('form'),
				titleelem;

			if (parentform.length > 0) {
				titleelem = parentform.find('input[data-autosave]');
				if (titleelem.length > 0) {
					return titleelem.get(0);
				}
				else {
					return parentform.get(0).elements['title'] || false;
				}
			}

			return false;
		}

		this.autosave_save_data = function(key)
		{
			if (typeof vBulletinStorage != 'undefined' && this.autosave_title)
			{
				var titletxt = jQuery(this.autosave_title).val();
				if (titletxt)
				{
					if (vBulletinStorage.webstoragetype != 3 && window.LZString)
					{
						titletxt = LZString.compressToUTF16(titletxt);
					}

					vBulletinStorage.setItem(key + '_title', titletxt);
				}
				else {
					this.autosave_remove_data(key);
				}
			}
		};

		this.autosave_restore_data = function(key)
		{
			if (typeof vBulletinStorage != 'undefined' && this.autosave_title)
			{
				var titletxt = vBulletinStorage.getItem(key + '_title');
				if (titletxt)
				{
					if (vBulletinStorage.webstoragetype != 3 && window.LZString)
					{
						titletxt = LZString.decompressFromUTF16(titletxt);
					}

					jQuery(this.autosave_title).val(titletxt);
				}
			}
		};

		this.autosave_remove_data = function(key)
		{
			if (typeof vBulletinStorage != 'undefined')
			{
				vBulletinStorage.removeItem(key + '_title');
			}
		};
	}

	// =============================================================================
	// Private functions
	// =============================================================================
	/**
	* Builds an object that filters incoming HTML content by transforming and
	* deleting disallowed elements, attributes, classes and styles in CKEDITOR
	*/
	function content_filter()
	{
		var tags = {};

		for (var tag in CKEDITOR.dtd)
		{
			if (tag.charAt(0) != '$' && tag != 'div'  && tag != 'span')
			{
				tags[tag] = 1;
			}
		}

		var filter = {
			// default filters fron CKEDITOR.dtd
			$1: {
				elements: tags,
				attributes: true,
				styles: false,
				classes: false
			},
			// span{color}
			$2: {
				elements: { span: 1 },
				attributes: false,
				styles: { color: 1 },
				classes: false
			}
		};

		return filter;
	}

	// gentlemen, start your engines...
	this.init();
}

/**
* Prompt users to leave/stay when there is editor content
*/
function promp_leave_or_stay(evt)
{
	var isformsubmit = false,
		istextchange = false;

	for (var editorid in vB_Editor)
	{
		if (vB_Editor[editorid].submitready)
		{
			isformsubmit = true;
			break;
		}

		if (vB_Editor[editorid].check_dirty())
		{
			istextchange = true;
		}
	}

	if (!isformsubmit && istextchange)
	{
		// Cancel the event as stated by the standard.
		evt.preventDefault();
		// Chrome requires returnValue to be set.
		evt.returnValue = vbphrase['leave_before_submit'];
	}
}

if (window.addEventListener)
{
	window.addEventListener('beforeunload', promp_leave_or_stay, false);
}
else
{
	window.attachEvent('onbeforeunload', promp_leave_or_stay);
}
