/*======================================================================*\
|| #################################################################### ||
|| # vBulletin [#]version[#]
|| #################################################################### ||
\*======================================================================*/

'use strict'

/**
* Class LazyLoad
*
* @var	object	Element that contains all LazyLoad images
* @var	object	Lazyload options
*/

function LazyLoad(node, options) {

	var defaults = {
		src: 'data-src',
		srcset: 'data-srcset',
		selectclass: 'lazyload',
		rootmargin: '0px 0px 0px 0px'
	};

	// Private functions

	var show_image = function(img, index) {
		var src = img.getAttribute(self.settings.src);
		var srcset = img.getAttribute(self.settings.srcset);

		switch (img.tagName.toUpperCase())
		{
			case 'PICTURE':
				var sources = img.getElementsByTagName('SOURCE');
				for (var i = 0; i < sources.length; i++) {
					srcset = sources[i].getAttribute(self.settings.srcset);
					if (srcset) {
						sources[i].setAttribute('srcset', srcset);
						sources[i].setAttribute(self.settings.srcset, '');
					}
				}
				var elem = img.getElementsByTagName('IMG')[0];
				elem.addEventListener('load', image_onload, { once: true });
				src = elem.getAttribute(self.settings.src);
				srcset = elem.getAttribute(self.settings.srcset);
				if (src) {
					elem.src = src;
					elem.setAttribute(self.settings.src, '');
				}
				if (srcset) {
					elem.setAttribute('srcset', srcset);
					elem.setAttribute(self.settings.srcset, '');
				}
				break;
			case 'IMG':
				img.addEventListener('load', image_onload, { once: true });
				if (src) {
					img.src = src;
					img.setAttribute(self.settings.src, '');
				}
				if (srcset) {
					img.srcset = srcset;
					img.setAttribute(self.settings.srcset, '');
				}
				break;
			default:
				img.style.backgroundImage = 'url(' + src + ')';
		}
	};

	var image_onload = function(evt) {
		var img = evt.target;
		if (jQuery(img).hasClass(self.settings.selectclass)) {
			jQuery(img).removeClass(self.settings.selectclass);
		}
		else if (jQuery(img).parent().hasClass(self.settings.selectclass)) {
			jQuery(img).parent().removeClass(self.settings.selectclass);
		}
	};

	var clean_list = function() {
		var i = self.imagelist.length;
		while (i--) {
			if (!jQuery(self.imagelist[i]).hasClass(self.settings.selectclass)) {
				self.imagelist.splice(i, 1);
			}
		}
	};

	var init = function(elems) {

		/* Without observers load everything and bail out early. */
		if (!('IntersectionObserver' in window)) {
			jQuery.each(elems, function (index, img) {
				show_image(img);
			});
			return;
		}

		if (self.observer == null) {

			var config = {
				root: null,
				rootMargin: self.settings.rootmargin,
				threshold: [0]
			};

			self.observer = new IntersectionObserver(function(entries) {
				entries.forEach(function (entry) {
					// Edge bug https://developer.microsoft.com/en-us/microsoft-edge/platform/issues/12156111/
					// if (entry.isIntersecting) {
					if (entry.intersectionRatio > 0) {
						self.observer.unobserve(entry.target);
						show_image(entry.target);
					}
				});
				clean_list();
			}, config);
		}

		/* Observe each image and add it to list */
		jQuery.each(elems, function (index, img) {
			self.observer.observe(img);
			self.imagelist.push(img);
		});
	};

	// Public functions

	this.observe = function(node, options) {
		options = jQuery.extend(this.settings, options || {});
		init(node.querySelectorAll(options.selector));
	}

	this.load_and_destroy = function () {
		if (!this.settings) {
			return;
		}
		this.load_images();
		this.destroy();
	}

	this.load_images = function () {
		if (!this.settings) {
			return;
		}

		this.imagelist.forEach(function(img) {
			self.observer.unobserve(img);
			show_image(img);
		});
		clean_list();
	}

	this.destroy = function () {
		if (!this.settings) {
			return;
		}
		this.observer.disconnect();
		this.observer = null;
		this.settings = {};
	}

	// Public variables

	var self = this;
	this.settings = jQuery.extend(defaults, options || {});
	this.settings.selector = '.' + this.settings.selectclass;
	this.observer = null;
	this.imagelist = [];

	return init(node.querySelectorAll(this.settings.selector));
};

/**
* Looks for images/pictures for lazy loading
*
* @param	mixed	Element/elementid containing images/pictures
*/
function init_lazy_images(elem, options) {

	var node;

	if (typeof elem == 'string') {
		node = fetch_object(elem);
	}
	else if (elem.target) {
		node = elem.target;
	}
	else if (elem == null) {
		node = document;
	}
	else {
		node = elem;
	}

	if (typeof vB_LazyLoad == 'undefined') {
		window.vB_LazyLoad = new LazyLoad(node, options);
	}
	else {
		vB_LazyLoad.observe(node, options);
	}
}
