'use strict';

function vB_WebStorage(type, edays)
{
	/**
	* Single instance
	*/
	if (constructor.prototype.instance)
	{
		return constructor.prototype.instance;
	}
	constructor.prototype.instance = this;

	/**
	* Variables
	*/
	this.webstoragetype = type || 0;

	// Expire content after # days
	var expiredays = parseInt(edays) || 7;
	// Check if web storage is available
	var webStorage = getWebStorage(this.webstoragetype);
	if (!webStorage) return;

	/* Ajax Storage */
	function ajaxStorage()
	{
		var Data = {};

		this.length = 0;

		this.key = function(n)
		{
			var keys = Object.keys(Data);
			return (!!keys[n]) ? keys[n] : null;
		};

		this.getItem = function(key)
		{
			return Data.hasOwnProperty(key) ? Data[key] : undefined;
		};

		this.setItem = function(key, value)
		{
			if (!key) return;

			Data[key] = String(value);

			this.length = Object.keys(Data).length;
			var p = false;
			console.log('Auto saved.');
		};

		this.removeItem = function(key)
		{
			this.length = Object.keys(Data).length;
			return delete Data[key];
		};

		this.clear = function()
		{
			Data = {};
			this.length = 0;
			return undefined;
		};

		// Send Data to the back-end to be stored
		var lastinterval = new Date();
		function saveData()
		{
			var content = {}, aux;

			for (var key in Data)
			{
				try
				{
					aux = JSON.parse(Data[key]);
					aux.updated = new Date(aux.updated);
				}
				catch (e)
				{
					continue;
				}
				if (aux.updated > lastinterval)
				{
					content[key] = PHP.urlencode(aux.data);
				}
			}

			if (Object.keys(content).length > 0)
			{
				jQuery.ajax({
					url: 'ajax.php?do=autosave',
					method: 'POST',
					data: {
						'do': 'autosave',
						content: content,
						securitytoken: SECURITYTOKEN
					},
					dataType: 'json',
					timeout: 20000,
				})
				.then(function(response) {
					console.log(response);
				})
				.fail(function(err) {
					console.error(err.toString())
				});
			}

			lastinterval = new Date();
		};
		var intervalid = setInterval(saveData, 60000);
	}

	function getWebStorage(storagetype)
	{
		var item = 'localStoragePolyfill';
		try
		{
			switch (storagetype)
			{
				// DB Storage thru Ajax
				case 3:
					return new ajaxStorage();
				// Session Storage
				case 2:
					window.sessionStorage.setItem(item, item);
					window.sessionStorage.removeItem(item);
					return window.sessionStorage;
				// Local Storage (default)
				case 1:
				default:
					window.localStorage.setItem(item, item);
					window.localStorage.removeItem(item);
					return window.localStorage;
			}
		}
		catch (e)
		{
			console.error(e.toString());
			return false;
		}
	}

	function getItemObject(key)
	{
		var value = webStorage.getItem(key);

		try
		{
			value = JSON.parse(value);
		}
		catch (e)
		{
			value = null;
			if (e.name != 'SyntaxError')
			{
				console.error(e);
			}
		}

		return value;
	}

	this.length = 0;

	this.key = function(n)
	{
		return webStorage.key(n);
	};

	this.getItem = function(key)
	{
		var value = getItemObject(key);
		return value && value.data ? value.data : (value ? value : null);
	};

	this.setItem = function(key, value)
	{
		if (!key || !value) return;

		value = JSON.stringify({
			data: value,
			updated: Date.now()
		});
		webStorage.setItem(key, value);

		this.length = webStorage.length;
	};

	this.removeItem = function(key)
	{
		webStorage.removeItem(key);
		this.length = webStorage.length;
	};

	this.clear = function()
	{
		if (typeof webStorage == 'object' && webStorage.constructor.name == 'ajaxStorage')
		{
			webStorage.clear();
		}
		else
		{
			// clear only vB prefixed keys
			var keys = Object.keys(webStorage), i = keys.length;
			while (i--)
			{
				if (keys[i].startsWith(COOKIE_PREFIX))
				{
					this.removeItem(keys[i]);
				}
			}
		}
		this.length = 0;
	};

	// Remove expired keys
	(function() {
		// Clear only vB prefixed keys
		var keys = Object.keys(webStorage).filter(function(k){
			return k.startsWith(COOKIE_PREFIX);
		});
		// Expire items last updated over expiredays
		var expired = new Date();
		expired.setTime(expired.getTime() - (1000 * 86400 * expiredays));
		// Finally remove expired keys
		for (var i = 0, item; i < keys.length; i++)
		{
			item = getItemObject(keys[i]);
			if (item && item.updated <= expired)
			{
				webStorage.removeItem(keys[i]);
				console.log("Item Expired :: %s", keys[i]);
			}
		}
	})();
}

function set_autoload(items)
{
	if (typeof vBulletinStorage !== 'undefined')
	{
		var value;

		for (var key in items)
		{
			value = jQuery('<div/>').html(items[key]).text();
			vBulletinStorage.setItem(key, value);
		}
	}
}
